@@ -33,9 +33,18 @@ class WithContext(object):
 class DoxyCPPClassObject(cpp.CPPClassObject):
     __bases = []
 
+    @property
+    def display_object_type(self):
+        # override because we also have the 'interface' type
+        assert self.objtype in ('class', 'struct', 'interface')
+        # TODO: remove this if it should be rendered as 'interface' as well
+        if self.objtype == 'interface':
+            return 'class'
+        return self.objtype
+
     def parse_definition(self, parser):
         # add the base classes
-        ast = parser.parse_declaration("class")
+        ast = parser.parse_declaration("class", "class")
 
         bases = []
 
@@ -74,10 +83,10 @@ class DomainDirectiveFactory(object):
         'friend': (cpp.CPPFunctionObject, 'function'),
         'signal': (cpp.CPPFunctionObject, 'function'),
         'slot': (cpp.CPPFunctionObject, 'function'),
-        'enum': (cpp.CPPTypeObject, 'type'),
+        'enum': (cpp.CPPEnumObject, 'enum'),
         'typedef': (cpp.CPPTypeObject, 'type'),
         'using': (cpp.CPPTypeObject, 'type'),
-        'union': (cpp.CPPTypeObject, 'type'),
+        'union': (cpp.CPPUnionObject, 'union'),
         'namespace': (cpp.CPPTypeObject, 'type'),
         'enumvalue': (cpp.CPPEnumeratorObject, 'enumerator'),
         'define': (c.CObject, 'macro')
@@ -171,11 +180,12 @@ def get_param_decl(param):
     def to_string(node):
         """Convert Doxygen node content to a string."""
         result = []
-        for p in node.content_:
-            value = p.value
-            if not isinstance(value, six.text_type):
-                value = value.valueOf_
-            result.append(value)
+        if node is not None:
+            for p in node.content_:
+                value = p.value
+                if not isinstance(value, six.text_type):
+                    value = value.valueOf_
+                result.append(value)
         return ' '.join(result)
 
     param_type = to_string(param.type_)
@@ -325,19 +335,11 @@ class SphinxRenderer(object):
 
         return '::'.join(names)
 
-    def insert_template_node(self, signature, decl):
-        """Creates a node for the ``template <...>`` part of the declaration."""
+    def create_template_prefix(self, decl):
         if not decl.templateparamlist:
-            return None
-        template = 'template '
-        nodes = [self.node_factory.desc_annotation(template, template), self.node_factory.Text('<')]
-        nodes.extend(self.render(decl.templateparamlist))
-        nodes.append(self.node_factory.Text(">"))
-
-        signode = type(signature)()
-        signode.extend(nodes)
-
-        signature.parent.insert(0, signode)
+            return ""
+        nodes = self.render(decl.templateparamlist)
+        return 'template<' + ''.join(n.astext() for n in nodes) + '>'
 
     def run_domain_directive(self, kind, names, augment=None):
         domain_directive = DomainDirectiveFactory.create(
@@ -477,13 +479,16 @@ class SphinxRenderer(object):
 
         def render_signature(file_data, doxygen_target, name, kind):
             # Defer to domains specific directive.
-            self.context.directive_args[1] = [self.get_fully_qualified_name()]
+
+            templatePrefix = self.create_template_prefix(file_data.compounddef)
+            arg = "%s %s" % (templatePrefix, self.get_fully_qualified_name())
 
             if kind in ('class', 'struct'):
                 augment = dict(bases=file_data.compounddef.basecompoundref)
             else:
                 augment = None
 
+            self.context.directive_args[1] = [arg]
             nodes = self.run_domain_directive(kind, self.context.directive_args[1], augment=augment)
             rst_node = nodes[1]
 
@@ -494,8 +499,6 @@ class SphinxRenderer(object):
             # from "class " to the correct kind which can be "class " or "struct ".
             finder.declarator[0] = self.node_factory.desc_annotation(kind + ' ', kind + ' ')
 
-            # Check if there is template information and format it as desired
-            self.insert_template_node(finder.declarator, file_data.compounddef)
             rst_node.children[0].insert(0, doxygen_target)
             return nodes, finder.content
 
@@ -707,10 +710,15 @@ class SphinxRenderer(object):
         nodelist = self.render_iterable(node.content)
         nodelist.extend(self.render_iterable(node.images))
 
-        # Returns, user par's, etc
-        definition_nodes = self.render_iterable(node.simplesects)
-        # Parameters/Exceptions
-        definition_nodes.extend(self.render_iterable(node.parameterlist))
+        if self.project_info.order_parameters_first():
+            # Order parameters before simplesects, which mainly are return/warnings/remarks
+            definition_nodes = self.render_iterable(node.parameterlist)
+            definition_nodes.extend(self.render_iterable(node.simplesects))
+        else:
+            # Returns, user par's, etc
+            definition_nodes = self.render_iterable(node.simplesects)
+            # Parameters/Exceptions
+            definition_nodes.extend(self.render_iterable(node.parameterlist))
 
         if definition_nodes:
             definition_list = self.node_factory.definition_list("", *definition_nodes)
@@ -937,10 +945,10 @@ class SphinxRenderer(object):
         decorator around the generic render function.
         Render all the children depth-first. """
         """ Call the wrapped render function. Update the nesting level for the enumerated lists. """
-        if node.node_subtype is "itemized":
+        if node.node_subtype == "itemized":
             val = self.render_iterable(node.listitem)
             return self.render_unordered(children=val)
-        elif node.node_subtype is "ordered":
+        elif node.node_subtype == "ordered":
             self.nesting_level += 1
             val = self.render_iterable(node.listitem)
             self.nesting_level -= 1
@@ -984,8 +992,11 @@ class SphinxRenderer(object):
             param = self.context.mask_factory.mask(param)
             param_decl = get_param_decl(param)
             param_list.append(param_decl)
-        signature = '{0}({1})'.format(get_definition_without_template_args(node),
-                                      ', '.join(param_list))
+        templatePrefix = self.create_template_prefix(node)
+        signature = '{0}{1}({2})'.format(
+            templatePrefix,
+            get_definition_without_template_args(node),
+            ', '.join(param_list))
 
         # Add CV-qualifiers.
         if node.const == 'yes':
@@ -1001,6 +1012,10 @@ class SphinxRenderer(object):
         elif node.refqual == 'rvalue':
             signature += '&&'
 
+        # Add `= 0` for pure virtual members.
+        if node.virt == 'pure-virtual':
+            signature += '= 0'
+
         self.context.directive_args[1] = [signature]
 
         nodes = self.run_domain_directive(node.kind, self.context.directive_args[1])
@@ -1012,13 +1027,7 @@ class SphinxRenderer(object):
         # Insert Doxygen target into the first signature node.
         rst_node.children[0].insert(0, self.create_doxygen_target(node))
 
-        # Add `= 0` for pure virtual members.
-        if node.virt == 'pure-virtual':
-            finder.declarator.append(self.node_factory.Text(' = 0'))
-
         finder.content.extend(self.description(node))
-
-        self.insert_template_node(finder.declarator, node)
         return nodes
 
     def visit_define(self, node):
@@ -1039,9 +1048,9 @@ class SphinxRenderer(object):
 
     def visit_enum(self, node):
         # Sphinx requires a name to be a valid identifier, so replace anonymous enum name of the
-        # form @id generated by Doxygen with "anonymous".
+        # form @id generated by Doxygen with "@anon_id".
         name = self.get_fully_qualified_name()
-        declaration = name.replace("@", "__anonymous") if node.name.startswith("@") else name
+        declaration = name.replace("@", "@anon_") if node.name.startswith("@") else name
 
         description_nodes = self.description(node)
         name = self.node_factory.emphasis("", self.node_factory.Text("Values:"))
@@ -1049,22 +1058,7 @@ class SphinxRenderer(object):
         description_nodes.append(title)
         enums = self.render_iterable(node.enumvalue)
         description_nodes.extend(enums)
-
-        def update_signature(signature, obj_type):
-            first_node = signature.children[0]
-            if isinstance(first_node, self.node_factory.desc_annotation):
-                # Replace "type" with "enum" in the signature. This is needed because older
-                # versions of Sphinx cpp domain didn't have an enum directive and we use a type
-                # directive instead.
-                first_node[0] = self.node_factory.Text("enum ")
-            else:
-                # If there is no "type" annotation, insert "enum".
-                first_node.insert(0, self.node_factory.desc_annotation("enum ", "enum "))
-            if node.name.startswith("@"):
-                signature.children[1][0] = self.node_factory.strong(text="[anonymous]")
-
-        return self.render_declaration(node, declaration, description_nodes,
-                                       update_signature=update_signature)
+        return self.render_declaration(node, declaration, description_nodes)
 
     def visit_typedef(self, node):
         declaration = get_definition_without_template_args(node)
@@ -1075,6 +1069,10 @@ class SphinxRenderer(object):
             declaration = declaration[len(typedef):]
         elif declaration.startswith(using):
             declaration = declaration[len(using):]
+            # remove the spurious "typedef " on right hand side added
+            # by Doxygen observed when type is declared inside a namespace
+            # See examples/specific/using_in_ns
+            declaration = declaration.replace(" = typedef ", " = ")
             obj_type = "using"
 
         def update_signature(signature, obj_type):
@@ -1089,8 +1087,9 @@ class SphinxRenderer(object):
         return self.render_declaration(node, declaration, objtype=obj_type,
                                        update_signature=update_signature)
 
-    def update_signature_with_initializer(self, signature, node):
+    def make_initializer(self, node):
         initializer = node.initializer
+        signature = []
         if initializer:
             render_nodes = self.render(initializer)
             # Do not append separators for paragraphs.
@@ -1100,24 +1099,23 @@ class SphinxRenderer(object):
                     separator += '= '
                 signature.append(self.node_factory.Text(separator))
             signature.extend(render_nodes)
+        return ''.join(n.astext() for n in signature)
 
     def visit_variable(self, node):
         declaration = get_definition_without_template_args(node)
         enum = 'enum '
         if declaration.startswith(enum):
             declaration = declaration[len(enum):]
-
-        def update_signature(signature, obj_type):
-            self.update_signature_with_initializer(signature, node)
-        return self.render_declaration(node, declaration, update_signature=update_signature)
+        declaration += self.make_initializer(node)
+        return self.render_declaration(node, declaration)
 
     def visit_enumvalue(self, node):
         def update_signature(signature, obj_type):
-            # Remove "class" from the signature. This is needed because Sphinx cpp domain doesn't
-            # have an enum value directive and we use a class directive instead.
+            # TODO: should the prefix still be removed after Sphinx supoprts enumerators?
             signature.children.pop(0)
-            self.update_signature_with_initializer(signature, node)
-        return self.render_declaration(node, objtype='enumvalue', update_signature=update_signature)
+        declaration = self.get_fully_qualified_name() + self.make_initializer(node)
+        return self.render_declaration(node, declaration=declaration,
+                                       objtype='enumvalue', update_signature=update_signature)
 
     def visit_param(self, node):
 
